classdef ls2 < handle
   %LS2 searches directories recursively.
   %    LS2 searches directories and shows directory listings recursively.  
   %    
   %    LS2 has a command-line interface and it allows user to perform 
   %    different kinds of listings or searches.  
   %    
   %    IN:
   %        path1, path2,...pathN -  directories to include in a seach [default: cwd]
   %        -h, --help            -  show help message
   %        -H, --hidden          -  show hidden files [default: off]  
   %        -fd DATE, 
   %        --filedate=DATE      -  filedate filter
   %                                All the files less than a specified
   %                                date 
   %                                will be filtered. You can reverse the filtering 
   %                                direction by placing the '>' mark in front of 
   %                                the DATE. In this case the program 
   %                                will filter all the files larger than a 
   %                                specified value. You must define the
   %                                date in a format dd.mm.yyyy.
   %        -fs SIZE,  
   %        --filesize=SIZE      -  filesize filter
   %                                All the files less than a specified size 
   %                                will be filtered.You can reverse the filtering 
   %                                direction by placing the > mark in front of 
   %                                the SIZE. In this case the program will filter all 
   %                                the files larger than a specified value.
   %                                You can also speficy the unit for 
   %                                the file size. Possible units are B,
   %                                KB, MB and GB.
   %        -gd N,
   %        --getdirectory=N     -  pick a directory from the dialog.
   %                                You can define the number of directories
   %                                you would like to include in a search by 
   %                                specifing the N value. 
   %
   %        -m, --modified       -  show last modified date/time [default: off]
   %        -o ORDER,
   %        --order=ORDER        -  order search by name, n, modified, m,
   %                                size, s [default: name]
   %        -r, --recursive      -  recurse into subdirectories [default: off]
   %        -s, --sizes          -  show sizes [default: off]
   %        -S SEARCHTEXT,
   %         --search=SEARCHTEXT -  search specific files   
   %                                When searching a particular file(s), you 
   %                                can define a search text in a form of str*, 
   %                                *str and str. Where str is a string 
   %                                which will be contained in a file's name.  
   %                                It's possible to use the wildcard character 
   %                                "*" at the end of a string or at the beginning 
   %                                of a string. If the asterisk operator 
   %                                is used at the end of string, a file name 
   %                                is assumed to start with a specified string. 
   %                                Consistently, if the asterisk operator 
   %                                is used at the beginning of a string, 
   %                                a file name is assumed to end with a specified
   %                                string. On the other hand a mere string 
   %                                without the asterisk operator is searched 
   %                                throughout the file name. The search is 
   %                                done based on case-insensitivity. All the 
   %                                matched files will be highlighted in red.
   %        -su, --suppress      -  suppress printing [default: off]
   % 
   %    LS2 class offers two public methods getListingData and doSearch. 
   %    getListingData method returns results from the search in a structure 
   %    array. This structure encompasses fields which are name, date, bytes,
   %    isdir and datenum. The name field consists of directory and file names.
   %    Directories and files are represented as a relative paths to the given
   %    input directory.   
   %    doSearch method runs a search. It resets all the properties before
   %    performing the search.
   
   %    Examples:    
   %        ls2('-m','-s')
   %        ls2('-m','-s','--hidden','D:\data\...')
   %        ls2('D:\data\...','D:\data\...','-m','-s','-r','-o s')
   %        ls2('-gd 2','-s','-m','-o s')
   %        ls2('--getdirectory=2','-s','-m','--order=m')
   %        ls2('-gd','-s','-m','-fs 5 kb','-o s')
   %        ls2('-gd','-s','-m','--filedate=>21.09.2011','-o s')
   %        ls2('-gd','-s','-m','-fd >21.09.2011','-fs 15 kb','-o s','-r')
   %        ls2('-gd 2','-s','-m','-o s','--search=*.pdf *.html *.xlsx')
   %        
   %        lsObj = ls2('-r','-su','-gd','-S *.m');    
   %        d = lsObj.getListingData;
   %        files = {d(~[d.isdir]).name};
   %        lsObj.doSearch('-r','-su','-gd','-S *.cpp *.h make*')
   %        d2 = lsObj.getListingData;
   %
   %    See also: ls, dir, ls_mod

   %    Author: Mikko Leppänen 27/01/2013
   % 
   %    Version history:
   %        27/01/2013 - first release
   %        15/02/2013 - second release
   %            Files and directories are returned as a relative paths.
   %        16/02/2013 - third release
   %            Directories are excluded from output structure array when 
   %            searching specific file specs and/or enabling file size/date filter.
   %            This will speed up the search process ~2x. 
   
   
    properties(Access=private)
        showModified = false;
        orderType = 'name';
        recurse = false;
        showSizes = false;
        showHidden = false;
        searchText = '';
        isSizeFilterOn = false;
        filterSizeField = '';
        filterSize = 0;
        filterDir = containers.Map({'size','date'},{'LT','LT'});
        fileSize = 0;
        fileSizeType = 'KB';
        isDateFilterOn = false;
        filterDateField = '';
        filterDate;
        suppressPrint = false;
        dirCount = 0;
        fileCount = 0;
        totSize = 0;
        nameFilter = {};
        nameFilterEnd = {};
        nameFilterStart = {};
        data;
        mainPath;
        currentPath = '';
        paths = {};
        listingData = struct('name',{},'date',{},...
                             'bytes',[],'isdir',[],'datenum',[]);
    end
    
    methods
        function obj = ls2(varargin) % class constructor
            clc;
            try 
                parseInputs(obj,varargin);
            catch ME
                error(ME.identifier,ME.message)
            end
            executeListing(obj);
                
        end  
        
        function data = getListingData(obj)
            data = obj.listingData;
        end
        
        function doSearch(obj,varargin)
            resetProperties(obj);
            clc;
            try 
                parseInputs(obj,varargin);
            catch ME
                error(ME.identifier,ME.message)
            end
            executeListing(obj);
        end
    end
    
    methods(Access=private) 
        function parseInputs(obj,varargin)
           if isempty(varargin)
               obj.currentPath = pwd; 
           end
           if ismember('-h',varargin{:}) || ismember('--help',varargin{:})
               help ls2
               return
           end
           for k = 1:numel(varargin{:})
              if ischar(varargin{1}{k})
                  [~, opts] = regexp(varargin{1}{k},['^\-(?<option>\w{1,2})(?<value>(?=\s{1}).+)?|',...
                                            '^\-\-(?<option>\w+)(?<value>(?=[=]{1}).+)?|^(?<directory>.+)'],'match','once','names');
                  if isempty(opts)
                     continue
                  else
                     if ~isempty(opts.directory) && isdir(opts.directory)
                         obj.paths{end+1} = opts.directory;
                     end
                     if ~isempty(opts.option)
                         switch opts.option
                             case {'gd','getdirectory'}
                                 if ~isempty(opts.value)
                                     numOfPaths = regexpi(opts.value(2:end),'^(\d+)','match','once');
                                     try 
                                        validateattributes(str2num(numOfPaths),{'numeric'},{'finite','scalar',...
                                                                                'nonempty','nonnegative'});
                                         for ii = 1:str2num(numOfPaths)
                                             if isempty(obj.currentPath)
                                                obj.paths{ii} = uigetdir(pwd,'Pick a Directory');
                                             else
                                                obj.paths{ii} = uigetdir(obj.currentPath,'Pick a Directory');
                                             end
                                         end
                                     catch ME %!>x?
                                         if isempty(obj.currentPath)
                                            obj.paths{end+1} = uigetdir(pwd,'Pick a Directory');
                                         else 
                                            obj.paths{end+1} = uigetdir(obj.currentPath,'Pick a Directory');
                                         end   
                                     end
                                 else
                                     if isempty(obj.currentPath)
                                        obj.paths{end+1} = uigetdir(pwd,'Pick a Directory');
                                     else
                                        obj.paths{end+1} = uigetdir(obj.currentPath,'Pick a Directory');
                                     end
                                 end

                             case {'m','modified'}
                                 obj.showModified = true;

                             case {'o','order'}
                                 if ~isempty(opts.value)
                                     switch opts.value(2:end)
                                         case {'name','n','modified','m','size','s'}
                                             obj.orderType = opts.value(2:end);
                                     end
                                 end

                             case {'r','recursive'}
                                 obj.recurse = true;

                             case {'s','sizes'}
                                 obj.showSizes = true;
                             case {'fs','filesize'}
                                 if ~isempty(opts.value)
                                     obj.isSizeFilterOn = true;
                                     obj.filterSizeField = opts.value(2:end);
                                 end

                             case {'H','hidden'}
                                 obj.showHidden = true;

                             case {'fd','filedate'}
                                 if ~isempty(opts.value)
                                     obj.isDateFilterOn = true;
                                     obj.filterDateField = opts.value(2:end);
                                 end

                             case {'S','search'}
                                  if ~isempty(opts.value)
                                    obj.searchText = opts.value(2:end);
                                  end
                                  
                             case {'su','suppress'}
                                 obj.suppressPrint = true;   
                         end
                     end
                    
                  end
           
              else 
                  continue
              end
           end
           if isempty(obj.paths)
               obj.paths{end+1} = pwd;
           end
        end

        function executeListing(obj)
            if ~isempty(obj.searchText)
                expr_1 = '(?<=\*)[\w-~\.]*[\w-~]*';
                expr_2 = '[\w-~\.]*(?=\*)';
                obj.nameFilterEnd = regexpi(obj.searchText,expr_1,'match');
                obj.nameFilterStart = regexpi(obj.searchText,expr_2,'match');
                obj.nameFilter = regexpi(regexprep(regexprep(regexprep(obj.searchText,...
                                 '(?<=\*)[\w-~\.]*[\w-~]*',''),'[\w-~\.]*(?=\*)',''),...
                                 '(\*)+',''),'[\w-~\.]*','match');
            else
                 obj.nameFilterEnd = {};
                 obj.nameFilterStart = {};
                 obj.nameFilter = {};
            end
            if obj.isSizeFilterOn
                if strcmp(obj.filterSizeField(1),'>')
                    obj.filterDir('size') = 'GT';
                else
                    obj.filterDir('size') = 'LT';
                end
                if ~isempty(regexpi(obj.filterSizeField,'(\d+\.?\d*)','match')) &&...
                    numel(regexpi(obj.filterSizeField,'(\d+\.?\d*)','match')) == 1
                    obj.filterSize = str2double(cell2mat(regexpi(obj.filterSizeField,...
                                               '(\d+\.?\d*)','match')));
                else
                    obj.filterSize = 0;
                end
                m = regexpi(obj.filterSizeField,'([a-zA-Z]{1,2}$)','match','once');
                if ~isempty(m)
                    switch upper(m)
                        case {'B','KB','MB','GB'}
                            obj.fileSizeType = upper(m);
                    end
                end
            else
                obj.filterSize = 0;
            end
            
            if obj.isDateFilterOn
                if strcmp(obj.filterDateField(1),'>')
                    obj.filterDir('date') = 'GT';
                else
                    obj.filterDir('date') = 'LT';
                end
                if ~isempty(regexp(obj.filterDateField,'(\d){1,2}(\.){1}(\d){1,2}(\.){1}(\d){4}','match'))
                    try
                        obj.filterDate = datenum(regexp(obj.filterDateField,...
                                        '(\d){1,2}(\.){1}(\d){1,2}(\.){1}(\d){4}','match'),'dd.mm.yyyy');
                    catch ME
                        warning(ME.identifier,ME.message)
                        warning('ignoring filtering date.')
                    end
                end
            end
            for ii = 1:numel(obj.paths)
                if ~isequal(obj.paths{ii},false) && isdir(obj.paths{ii})
                    obj.currentPath = obj.paths{ii};
                    obj.data = dir(obj.currentPath);
                    if ~isempty(obj.data)
                        dirnames = {obj.data([obj.data.isdir]).name};
                        filenames = {obj.data(~[obj.data.isdir]).name};
                        if obj.recurse && ~obj.suppressPrint
                            fprintf('\n')
                        end
                        obj.mainPath = obj.currentPath;
                        processlist(obj,filenames,dirnames)
                        if obj.recurse % recursive option on
                            if ~obj.showHidden
                                dirnames(~cellfun(@isempty,regexpi(dirnames,'^\.','match'))) = [];
                            else
                                dirnames(~cellfun(@isempty,regexpi(dirnames,'^\.$|^\.\.$','match'))) = [];
                            end
                            for n = 1:numel(dirnames)
                                obj.mainPath = fullfile(obj.currentPath,dirnames{n});
                                if isempty(obj.searchText)
                                    if ~obj.suppressPrint
                                        fprintf('\n\t%s:\n',fullfile(obj.currentPath,dirnames{n}))
                                    end
                                    recursivelisting(obj,fullfile(obj.currentPath,dirnames{n}))
                                else
                                    recursivelisting(obj,fullfile(obj.currentPath,dirnames{n}))
                                end
                            end
                            if ~obj.suppressPrint
                                if ~isempty(obj.searchText)
                                    fprintf('\n\t%d files, total %10G MB\n',obj.fileCount,obj.totSize/1e6);
                                else
                                    fprintf('\n\t%d files, %d directory, total %10G MB\n',obj.fileCount,obj.dirCount,obj.totSize/1e6);
                                end
                            end
                        end
                    end
                end
            end
            obj.dirCount = 0;
            obj.fileCount = 0;
            obj.totSize = 0;
            if ~obj.suppressPrint
                fprintf('\n')
            end
        end  
        
        function processlist(obj,files,dirs)
            % process directories and files according to input options and
            % print the results
            printdata = {};
            if ~isempty(dirs) && isempty(obj.searchText) &&...
               ~obj.isSizeFilterOn &&  ~obj.isDateFilterOn
                datesForDirs = {obj.data([obj.data.isdir]).datenum};
                datesForDirs = datesForDirs(:);
                if ismember(obj.orderType,{'m','modified'}) % order by last modified date
                    [~,ind1] = sort([obj.data([obj.data.isdir]).datenum],'descend');
                    dirs = dirs(ind1);
                    datesForDirs = datesForDirs(ind1);
                end
                if ~obj.showHidden % show hidden option off
                    datesForDirs(~cellfun(@isempty,regexpi(dirs,'^\.','match'))) = [];
                    dirs(~cellfun(@isempty,regexpi(dirs,'^\.','match'))) = [];
                else
                    datesForDirs(~cellfun(@isempty,regexpi(dirs,'^\.$|^\.\.$','match'))) = [];
                    dirs(~cellfun(@isempty,regexpi(dirs,'^\.$|^\.\.$','match'))) = [];
                end
                dirs = dirs(:);
                printdata = cell(numel(dirs),1);
                n = numel([obj.listingData.isdir]);
                if ~isempty(numel([obj.listingData.isdir])+(1:numel(dirs)))
                    for k = n+(1:numel(dirs))
                        obj.listingData(k,1).name = regexprep(fullfile(regexprep(obj.mainPath,...
                                                              regexprep(obj.currentPath,dirs{k-n},''),''),dirs{k-n}),...
                                                              '^[/ \\]?','');
                        obj.listingData(k,1).datenum = datesForDirs{k-n};
                        obj.listingData(k,1).date = datestr(datesForDirs{k-n},'dd-mmm-yyyy HH:MM:SS');
                        obj.listingData(k,1).bytes = 0;
                        obj.listingData(k,1).isdir = true;
                    end
                end
                if ~obj.suppressPrint
                    datesForDirs = cellfun(@datestr,datesForDirs,...
                    repmat({'dd-mmm-yyyy HH:MM:SS'},numel(datesForDirs),1),'uni',false);
                    for k = 1:numel(dirs)
                        if obj.showModified
                            printdata{k} = [sprintf('\t%-39s',datesForDirs{k}),sprintf('% s',dirs{k})];
                        else
                            printdata{k} = [sprintf('\t%-39s',''),sprintf('% s',dirs{k})];
                        end
                    end
                end
            end
            if ~isempty(files)   
                datesForFiles = {obj.data(~[obj.data.isdir]).datenum};
                bytesForFiles = {obj.data(~[obj.data.isdir]).bytes};
                % order cell arrays columnwise to reduce search time
                datesForFiles = datesForFiles(:);
                bytesForFiles = bytesForFiles(:);
                files = files(:);
                if ismember(obj.orderType,{'m','modified'})  % order by last modified date
                    [~,ind2] = sort([obj.data(~[obj.data.isdir]).datenum],'descend');
                    files = files(ind2);
                    datesForFiles = datesForFiles(ind2);
                    bytesForFiles = bytesForFiles(ind2);
                elseif ismember(obj.orderType,{'s','size'}) % order by size
                    [bytesForFiles,ind3] = sort([obj.data(~[obj.data.isdir]).bytes],'descend');
                    if isa(bytesForFiles,'double')
                        bytesForFiles = num2cell(bytesForFiles);
                    end
                    files = files(ind3);
                    datesForFiles = datesForFiles(ind3);
                end
                if ~obj.showHidden % show hidden option off
                    datesForFiles(~cellfun(@isempty,regexpi(files,'^\.','match'))) = [];
                    bytesForFiles(~cellfun(@isempty,regexpi(files,'^\.','match'))) = [];
                    files = files(cellfun(@isempty,regexpi(files,'^\.','match')));
                end
                inds = [];
                if ~isempty(obj.nameFilterEnd)
                    for k = 1:numel(obj.nameFilterEnd)
                        inds = [inds;find(~cellfun(@isempty,regexpi(files,...
                               ['[\w-~\.]*',regexprep(obj.nameFilterEnd{k},...
                               '\.','\\\.'),'$'],'match')))];
                    end
                end
                if ~isempty(obj.nameFilterStart)
                    for k = 1:numel(obj.nameFilterStart)
                        inds = [inds;find(~cellfun(@isempty,regexpi(files,...
                               ['^',regexprep(obj.nameFilterStart{k},...
                               '\.','\\\.'),'[\w-~\.]*'],'match')))];
                    end
                end
                if ~isempty(obj.nameFilter)
                    for k = 1:numel(obj.nameFilter)
                        inds = [inds;find(~cellfun(@isempty,regexpi(files,...
                               ['[\w-~\.]*',regexprep(obj.nameFilter{k},...
                               '\.','\\\.'),'[\w\-\~\.]*'],'match')))];
                    end
                end
                if ~isempty(obj.searchText)
                    inds = unique(inds);
                    files = files(inds);
                    datesForFiles = datesForFiles(inds);
                    bytesForFiles = bytesForFiles(inds);
                end
                printdata = [printdata;cell(numel(files),1)];
                if ~isempty(files) && ~obj.suppressPrint
                    datesForFiles = cellfun(@datestr,datesForFiles,...
                    repmat({'dd-mmm-yyyy HH:MM:SS'},numel(datesForFiles),1),'uni',false);
                    for k = numel(dirs)+(1:numel(files))
                        if obj.showModified % show modified date
                            printdata{k} = sprintf('\t%-23s',datesForFiles{k-numel(dirs)});
                        else
                            printdata{k} = sprintf('\t%-23s',' ');
                        end
                        if obj.showSizes % show sizes
                            printdata{k} = [printdata{k},sprintf('% 15s',num2str(bytesForFiles{k-numel(dirs)}))];
                        else
                            printdata{k} = [printdata{k},sprintf('% 15s','')];
                        end
                        printdata{k} = [printdata{k},sprintf(' % s',files{k-numel(dirs)})];
                    end
                end
                if obj.isSizeFilterOn
                    filInd = zeros(100,1);
                    sizefactor = 1e3;
                    switch obj.fileSizeType
                        case {'B'}
                            sizefactor = 1;
                        case {'MB'}
                            sizefactor = 1e6;
                        case {'GB'}
                            sizefactor = 1e9;
                    end
                    switch obj.filterDir('size')
                        case {'LT'}     
                            for it = numel(dirs)+(1:numel(bytesForFiles))
                                if bytesForFiles{it-numel(dirs)} < obj.filterSize*sizefactor;
                                    filInd(it-numel(dirs)) = it;
                                end
                            end
                        case {'GT'}
                            for it = numel(dirs)+(1:numel(bytesForFiles))
                                if bytesForFiles{it-numel(dirs)} > obj.filterSize*sizefactor;
                                    filInd(it-numel(dirs)) = it;
                                end
                            end
                    end
                    if any(filInd)
                        if ~obj.suppressPrint
                            printdata(filInd(filInd > 0)) = [];
                        end
                        files(filInd(filInd > 0)-numel(dirs)) = [];
                        bytesForFiles(filInd(filInd > 0)-numel(dirs)) = [];
                        datesForFiles(filInd(filInd > 0)-numel(dirs)) = [];
                    end
                end
                if obj.isDateFilterOn
                    filIndF = zeros(100,1);
                    filIndD = zeros(100,1);
                    switch obj.filterDir('date')
                        case 'LT'
                            for it = 1:numel(dirs)
                                try 
                                    if datenum(datesForDirs{it},'dd-mmm-yyyy HH:MM:SS') <...
                                        obj.filterDate
                                        filIndD(it) = it;
                                    end
                                catch ME
                                    %warning(ME.identifier,ME.message)
                                end
                            end
                            if ~isempty(files)
                                for it = numel(dirs)+(1:numel(bytesForFiles))
                                    try 
                                        if datenum(datesForFiles{it-numel(dirs)},'dd-mmm-yyyy HH:MM:SS') <...
                                            obj.filterDate
                                            filIndF(it-numel(dirs)) = it;
                                        end
                                    catch ME
                                        %warning(ME.identifier,ME.message)
                                    end
                                end
                            end
                        case 'GT'
                            for it = 1:numel(dirs)
                                try
                                    if datenum(datesForDirs{it},'dd-mmm-yyyy HH:MM:SS') >...
                                        obj.filterDate
                                        filIndD(it) = it;
                                    end
                                catch ME
                                    %warning(ME.identifier,ME.message)
                                end
                            end
                            if ~isempty(files)
                                for it = numel(dirs)+(1:numel(bytesForFiles))
                                    try 
                                        if datenum(datesForFiles{it-numel(dirs)},'dd-mmm-yyyy HH:MM:SS') >...
                                            obj.filterDate
                                            filIndF(it-numel(dirs)) = it;
                                        end
                                    catch ME
                                        %warning(ME.identifier,ME.message)
                                    end
                                end
                            end
                    end
                    if any(filIndF) && ~any(filIndD)
                        if ~obj.suppressPrint 
                            printdata(filIndF(filIndF > 0)) = [];
                        end
                        files(filIndF(filIndF > 0)-numel(dirs)) = [];
                        bytesForFiles(filIndF(filIndF > 0)-numel(dirs)) = [];
                    elseif any(filIndD)
                        if any(filIndF)
                            if ~obj.suppressPrint
                                printdata([filIndD(filIndD > 0);filIndF(filIndF > 0)]) = [];
                            end
                            files(filIndF(filIndF > 0)-numel(dirs)) = [];
                            bytesForFiles(filIndF(filIndF > 0)-numel(dirs)) = [];
                        else
                            if ~obj.suppressPrint
                                printdata(filIndD(filIndD > 0)) = [];
                            end
                        end
                        dirs(filIndD(filIndD > 0)) = [];
                    end
                end
                
            end
            ii = numel([obj.listingData.isdir]);
            if ~isempty(numel([obj.listingData.isdir])+(1:numel(files)))
                for k = ii+(1:numel(files))
                    obj.listingData(k,1).name = regexprep(fullfile(regexprep(obj.mainPath,...
                                                          obj.currentPath,''),files{k-ii}),...
                                                          '^[/ \\]?','');
                    obj.listingData(k,1).datenum = datesForFiles{k-ii};
                    obj.listingData(k,1).date = datestr(datesForFiles{k-ii},'dd-mmm-yyyy HH:MM:SS');
                    obj.listingData(k,1).bytes = bytesForFiles{k-ii};
                    obj.listingData(k,1).isdir = false;
                end
            end
            obj.fileCount = obj.fileCount + numel(files); % count files
            obj.dirCount = obj.dirCount + numel(dirs);
            if exist('bytesForFiles','var')
                obj.totSize = obj.totSize + sum(cell2mat(bytesForFiles)); % count sizes
            end
            if ~obj.suppressPrint
                if ~obj.recurse
                    fprintf('\n');
                    if ~isempty(obj.searchText)
                        fprintf('\t%s:\n',obj.currentPath);
                        fprintf(2,'%s\n',printdata{numel(dirs)+(1:numel(files))});
                        fprintf('\n\t%d files, total %10G MB\n',obj.fileCount, obj.totSize/1e6);
                    else
                        fprintf('%s\n',printdata{:});
                        fprintf('\n\t%d files, %d directory, total %10G MB\n',obj.fileCount,obj.dirCount,obj.totSize/1e6);
                    end
                else
                    if ~isempty(obj.searchText)
                        if ~isempty(files) 
                            fprintf('\n\t%s:\n',obj.mainPath);
                            fprintf(2,'%s\n',printdata{numel(dirs)+(1:numel(files))});
                        end
                    else
                        fprintf('%s\n',printdata{:});
                    end
                end
            end
        end
        
        function recursivelisting(obj,dirs)
            % list subdirectories recursively
            obj.data = dir(dirs);
            dirnames = {obj.data([obj.data.isdir]).name};
            filenames = {obj.data(~[obj.data.isdir]).name};
            obj.mainPath = dirs;
            processlist(obj,filenames,dirnames)
            if ~obj.showHidden
                dirnames(~cellfun(@isempty,regexpi(dirnames,'^\.','match'))) = [];
            else
                dirnames(~cellfun(@isempty,regexpi(dirnames,'^\.$|^\.\.$','match'))) = [];
            end
            for k = 1:numel(dirnames)
                obj.mainPath = fullfile(dirs,dirnames{k});
                if isempty(obj.searchText)
                    if ~obj.suppressPrint
                        fprintf('\n\t%s:\n',fullfile(dirs,dirnames{k}))
                    end
                    recursivelisting(obj,fullfile(dirs,dirnames{k}))
                else
                    recursivelisting(obj,fullfile(dirs,dirnames{k}))
                end
            end
        end
        
        function resetProperties(obj)
            obj.showModified = false;
            obj.orderType = 'name';
            obj.recurse = false;
            obj.showSizes = false;
            obj.showHidden = false;
            obj.searchText = '';
            obj.isSizeFilterOn = false;
            obj.filterSizeField = '';
            obj.filterSize = 0;
            obj.filterDir = containers.Map({'size','date'},{'LT','LT'});
            obj.fileSize = 0;
            obj.fileSizeType = 'KB';
            obj.isDateFilterOn = false;
            obj.filterDateField = '';
            obj.filterDate;
            obj.suppressPrint = false;
            obj.dirCount = 0;
            obj.fileCount = 0;
            obj.totSize = 0;
            obj.nameFilter = {};
            obj.nameFilterEnd = {};
            obj.nameFilterStart = {};
            obj.data;
            obj.mainPath;
            obj.currentPath = '';
            obj.paths = {};
            obj.listingData = struct('name',{},'date',{},...
                                     'bytes',[],'isdir',[],'datenum',[]);
        end 
    end 
end

